package gencode;

import java.util.Arrays;

/**
 * @Author Miracle Luna
 * @Date 2019/6/9 23:40
 * @Version 1.0
 */
public enum MysqlDataType {

    INT("int", "Integer", "java.lang.Integer"),
    INTEGER("integer", "Long", "java.lang.Long"),
    BIGINT("bigint", "Long", "java.lang.Long"),
    TINYINT("tinyint", "Integer", "java.lang.Integer"),
    SMALLINT("smallint", "Integer", "java.lang.Integer"),
    MEDIUMINT("mediumint", "Integer", "java.lang.Integer"),
    DOUBLE("double", "Double", "java.lang.Double"),
    DECIMAL("decimal", "BigDecimal", "java.math.BigDecimal"),
    VARCHAR("varchar", "String", "java.lang.String"),
    CHAR("char", "String", "java.lang.String"),
    TEXT("text", "String", "java.lang.String"),
    DATE("date", "Date", "java.util.Date"),
    DATETIME("datetime", "Date", "java.util.Date"),
    ERROR(null, null, null);
    private String key;

    private String value;

    private String column;

    private String javaPackage;

    /**
     * @param key
     * @param value
     */
    MysqlDataType(String key, String value, String javaPackage) {
        this.key = key;
        this.value = value;
        this.javaPackage = javaPackage;
    }

    public String getColumn() {
        return column;
    }

    public void setColumn(String column) {
        this.column = upperTable(column);
    }


    /**
     * 方法说明 :将首字母和带 _ 后第一个字母 转换成大写
     *
     * @return :String
     * @author :HFanss
     * @date :2018年5月31日下午9:52:19
     */
    public static String upperTable(String str) {
        // 字符串缓冲区
        StringBuffer sbf = new StringBuffer();
        // 如果字符串包含 下划线
        if (str.contains("_")) {
            // 按下划线来切割字符串为数组
            String[] split = str.split("_");
            // 循环数组操作其中的字符串
            for (int i = 0, index = split.length; i < index; i++) {
                // 递归调用本方法
                String upperTable = upperTable(split[i]);
                // 添加到字符串缓冲区
                sbf.append(upperTable);
            }
        } else {// 字符串不包含下划线
            // 转换成字符数组
            char[] ch = str.toCharArray();
            // 判断首字母是否是字母
            if (ch[0] >= 'a' && ch[0] <= 'z') {
                // 利用ASCII码实现大写
                ch[0] = (char) (ch[0] - 32);
            }
            // 添加进字符串缓存区
            sbf.append(ch);
        }
        // 返回
        return sbf.toString();
    }

    /**
     * @return Returns the key.
     */
    public String getKey() {
        return key;
    }

    /**
     * @return Returns the name.
     */
    public String getValue() {
        return value;
    }

    /**
     * @return Returns the name.
     */
    public String getJavaPackage() {
        return javaPackage;
    }

    /**
     * 根据Key得到枚举的Value
     * 普通for循环遍历，比较判断
     *
     * @param key
     * @return
     */
    public static MysqlDataType getEnumType(String key) {
        MysqlDataType[] alarmGrades = MysqlDataType.values();
        for (int i = 0; i < alarmGrades.length; i++) {
            if (alarmGrades[i].getKey().equals(key)) {
                return alarmGrades[i];
            }
        }
        return MysqlDataType.ERROR;
    }

    /**
     * 根据Key得到枚举的Value
     * 增强for循环遍历，比较判断
     *
     * @param key
     * @return
     */
    public static MysqlDataType getEnumType1(String key) {
        MysqlDataType[] alarmGrades = MysqlDataType.values();
        for (MysqlDataType alarmGrade : alarmGrades) {
            if (alarmGrade.getKey().equals(key)) {
                return alarmGrade;
            }
        }
        return MysqlDataType.ERROR;
    }

    /**
     * 根据Key得到枚举的Value
     * Lambda表达式，比较判断（JDK 1.8）
     *
     * @param key
     * @return
     */
    public static MysqlDataType getEnumType2(String key) {
        MysqlDataType[] alarmGrades = MysqlDataType.values();
        MysqlDataType result = Arrays.asList(alarmGrades).stream()
                .filter(alarmGrade -> alarmGrade.getKey().equals(key))
                .findFirst().orElse(MysqlDataType.ERROR);

        return result;
    }

    /**
     * 根据Key得到枚举的Value
     * Lambda表达式，比较判断（JDK 1.8）
     *
     * @param key
     * @return
     */
    public static MysqlDataType getEnumType3(String key) {
        return Arrays.asList(MysqlDataType.values()).stream()
                .filter(alarmGrade -> alarmGrade.getKey().equals(key))
                .findFirst().orElse(MysqlDataType.ERROR);
    }

    public static void main(String[] args) {
        String dataType = "tinyint";
        MysqlDataType enumType = MysqlDataType.getEnumType(dataType);
        System.out.println("第一种方式：普通for循环遍历，比较判断:" + enumType.getJavaPackage());
//        System.out.println("\n第二种方式：增强for循环遍历，比较判断 \n" + grade + ": " + MysqlDataType.getEnumType1(grade).getValue());
//        System.out.println("\n第三种方式：Lambda表达式，比较判断 \n" + grade + ": " + MysqlDataType.getEnumType2(grade).getValue());
//        System.out.println("\n第四种方式：Lambda表达式，比较判断 \n" + grade + ": " + MysqlDataType.getEnumType3(grade).getValue());
    }
}
